"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sl_env_vars_1 = require("../constants/sl-env-vars");
var original_module_loader_1 = require("../coverage-elements/original-module-loader");
var hits_collector_1 = require("./hits-collector");
var new_id_resolver_1 = require("../coverage-elements/new-id-resolver");
var HitsConverter = /** @class */ (function () {
    function HitsConverter(relativePathResolver, sourceMapData, projectRoot, logger) {
        this.relativePathResolver = relativePathResolver;
        this.sourceMapData = sourceMapData;
        this.projectRoot = projectRoot;
        this.logger = logger;
    }
    HitsConverter.prototype.convertHits = function (hitFilesData) {
        var _this = this;
        var methodUniqueIds = [];
        var branchUniqueIds = [];
        if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
            new original_module_loader_1.OriginalModuleLoader(hits_collector_1.HitsCollector.resolveGlobalCoverageObject(), this.logger).load();
        }
        hitFilesData.forEach(function (fileHit) {
            var relativePath = fileHit.filename;
            if (fileHit.shouldResolveRelativePath) {
                relativePath = _this.relativePathResolver.getRelativePath(fileHit.filename);
            }
            methodUniqueIds = methodUniqueIds.concat(_this.createMethodIds(fileHit.hitMethods, relativePath, fileHit.filename));
            branchUniqueIds = branchUniqueIds.concat(_this.createBranchIds(fileHit.hitBranches, relativePath, fileHit.filename));
        });
        return {
            branches: branchUniqueIds,
            methods: methodUniqueIds
        };
    };
    HitsConverter.prototype.createMethodIds = function (hitFunctions, relativePath, absolutePath) {
        var _this = this;
        var methodIds = [];
        hitFunctions.forEach(function (hit) {
            methodIds.push(_this.getMethodUniqueId(_this.getStartLoc(hit), relativePath, absolutePath));
            if (_this.getDecl(hit) && _this.getDeclStart(hit)) {
                methodIds.push(_this.getMethodUniqueId(_this.getDeclStart(hit), relativePath, absolutePath));
            }
        });
        return methodIds;
    };
    HitsConverter.prototype.getDeclStart = function (hit) {
        return hit.decl.start;
    };
    HitsConverter.prototype.getDecl = function (hit) {
        return hit.decl;
    };
    HitsConverter.prototype.getStartLoc = function (hit) {
        return hit.loc.start;
    };
    HitsConverter.prototype.getLeaveStartLoc = function (hit, leaveIdx) {
        return hit.branchData.locations[leaveIdx].start;
    };
    HitsConverter.prototype.createBranchIds = function (hitFunctions, relativePath, absolutePath) {
        var _this = this;
        var branchIds = [];
        hitFunctions.forEach(function (hit) {
            hit.hitLeaves.forEach(function (leaveIdx) {
                branchIds.push(_this.getBranchUniqueId(_this.getLeaveStartLoc(hit, leaveIdx), relativePath, absolutePath, leaveIdx));
            });
        });
        return branchIds;
    };
    HitsConverter.prototype.getMethodUniqueId = function (startLoc, relativePath, absolutePath) {
        var idParts = this.resolveIdParts(startLoc, relativePath, absolutePath);
        if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
            var uniqueIdKey = idParts.relativePath + "@" + this.formatLoc(idParts.start);
            var uniqueId = new_id_resolver_1.resolveNewId(uniqueIdKey, idParts.absolutePath, idParts.relativePath, hits_collector_1.HitsCollector.resolveGlobalCoverageObject(), this.logger);
            if (uniqueId) {
                return uniqueId;
            }
        }
        return this.buildMethodUniqueId(idParts);
    };
    HitsConverter.prototype.getBranchUniqueId = function (startLoc, relativePath, absolutePath, leaveIndex) {
        var uniqueId;
        var parts = this.resolveIdParts(startLoc, relativePath, absolutePath);
        if (sl_env_vars_1.SlEnvVars.isUseNewUniqueId()) {
            var uniqueIdKey = parts.relativePath + "|" + this.formatLoc(parts.start) + "|" + leaveIndex;
            uniqueId = new_id_resolver_1.resolveNewId(uniqueIdKey, parts.absolutePath, parts.relativePath, hits_collector_1.HitsCollector.resolveGlobalCoverageObject(), this.logger);
        }
        if (!uniqueId) {
            uniqueId = parts.relativePath + HitsConverter.BRANCH_ID_DEL + this.formatLoc(parts.start) + HitsConverter.BRANCH_ID_DEL + leaveIndex;
        }
        return uniqueId;
    };
    HitsConverter.prototype.resolveIdParts = function (startLoc, relativePath, absolutePath) {
        var sourcePosition = this.sourceMapData.getSourcePosition(relativePath, absolutePath, startLoc, this.projectRoot);
        return sourcePosition ? sourcePosition : { relativePath: relativePath, start: startLoc, absolutePath: absolutePath };
    };
    HitsConverter.prototype.buildMethodUniqueId = function (parts) {
        return parts.relativePath + HitsConverter.METHOD_ID_DEL + this.formatLoc(parts.start);
    };
    HitsConverter.prototype.formatLoc = function (loc) {
        return loc.line + ',' + loc.column;
    };
    HitsConverter.METHOD_ID_DEL = "@";
    HitsConverter.BRANCH_ID_DEL = "|";
    return HitsConverter;
}());
exports.HitsConverter = HitsConverter;
//# sourceMappingURL=hits-converter.js.map